@@ -142,8 +142,7 @@ xsltEvalXPathStringNs(xsltTransformContextPtr ctxt, xmlXPathCompExprPtr comp,
             ret = res->stringval;
 	    res->stringval = NULL;
 	} else {
-	    xsltPrintErrorContext(ctxt, NULL, NULL);
-	    xsltGenericError(xsltGenericErrorContext,
+	    xsltTransformError(ctxt, NULL, NULL,
 		 "xpath : string() function didn't return a String\n");
 	}
 	xmlXPathFreeObject(res);
@@ -225,7 +224,6 @@ xsltEvalTemplateString(xsltTransformContextPtr ctxt, xmlNodePtr node,
  * @ctxt:  the XSLT transformation context
  * @str:  the attribute template node value
  * @node:  the node hosting the attribute
- * @nsList:  the list of in-scope namespaces to use
  *
  * Process the given string, allowing to pass a namespace mapping
  * context and return the new string value.
@@ -269,7 +267,7 @@ xsltAttrTemplateValueProcessNode(xsltTransformContextPtr ctxt,
 		/*
 		 * TODO: keep precompiled form around
 		 */
-		if (nsList == NULL) {
+		if ((nsList == NULL) && (node != NULL)) {
 		    int i = 0;
 
 		    nsList = xmlGetNsList(node->doc, node);
@@ -425,7 +423,11 @@ xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
 	if (xmlStrEqual(cur->name, (const xmlChar *)"use-attribute-sets")) {
 	    xmlChar *in;
 
-	    in = xmlNodeListGetString(ctxt->document->doc, cur->children, 1);
+	    if (ctxt->document != NULL)
+	        in = xmlNodeListGetString(ctxt->document->doc,
+			                  cur->children, 1);
+	    else
+	        in = xmlNodeListGetString(NULL, cur->children, 1);
 	    if (in != NULL) {
 		xsltApplyAttributeSet(ctxt, ctxt->node, NULL, in);
 		xmlFree(in);
@@ -439,9 +441,13 @@ xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
 	ns = NULL;
 
     if (cur->children != NULL) {
-	xmlChar *in = xmlNodeListGetString(ctxt->document->doc,
-		                           cur->children, 1);
+	xmlChar *in;
 	xmlChar *out;
+	
+	if (ctxt->document != NULL)
+	    in = xmlNodeListGetString(ctxt->document->doc, cur->children, 1);
+	else
+	    in = xmlNodeListGetString(NULL, cur->children, 1);
 
 	/* TODO: optimize if no template value was detected */
 	if (in != NULL) {
