@@ -264,12 +264,15 @@ xsltDecimalFormatGetByName(xsltStylesheetPtr sheet, xmlChar *name)
 
     if (name == NULL)
 	return sheet->decimalFormat;
-    
-    for (result = sheet->decimalFormat->next;
-	 result != NULL;
-	 result = result->next) {
-	if (xmlStrEqual(name, result->name))
-	    break; /* for */
+
+    while (sheet != NULL) {
+	for (result = sheet->decimalFormat->next;
+	     result != NULL;
+	     result = result->next) {
+	    if (xmlStrEqual(name, result->name))
+		return result;
+	}
+	sheet = xsltNextImport(sheet);
     }
     return result;
 }
@@ -1980,7 +1983,14 @@ xsltParseStylesheetImportedDoc(xmlDocPtr doc) {
 	xsltFreeStylesheet(ret);
 	ret = NULL;
     }
-
+    if (ret != NULL) {
+	if (ret->errors != 0) {
+	    ret->doc = NULL;
+	    xsltFreeStylesheet(ret);
+	    ret = NULL;
+	}
+    }
+    
     return(ret);
 }
 
